// ----------------------------------------------------------------------------------
// Microsoft Developer & Platform Evangelism
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
// OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// ----------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
// ----------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Collections.ObjectModel;
using System.ComponentModel.DataAnnotations;
using System.Globalization;
using System.Text;
using PlanMyNight.Models;
using PlanMyNight.Models.Entities;
using PlanMyNight.ViewModels;

namespace PlanMyNight.Controllers
{
    [HandleError]
    public class HomeController : Controller
    {
        private const int DefaultPageSize = 5;

        private readonly IActivitiesRepository activitiesRepository;

        private IEnumerable<ActivityType> activityTypes;

        private IEnumerable<ActivityType> ActivityTypes
        {
            get
            {
                if (activityTypes == null)
                {
                    this.activityTypes = this.activitiesRepository.RetrieveActivityTypes();
                }

                return this.activityTypes;
            }
        }

        public HomeController() :
            this(new ActivitiesRepository())
        {
        }

        public HomeController(IActivitiesRepository activitiesRepository)
        {
            this.activitiesRepository = activitiesRepository;
        }


        public ActionResult Index()
        {
            this.InjectActivitySearchFields(this.ViewData, null);
            return View("Index");
        }


        public ActionResult About()
        {
            return View();
        }

        private void InjectActivitySearchFields(IDictionary<string, object> viewData, ActivitySearchCriteria searchCriteria)
        {
            var types = this.ActivityTypes.Select(o => new SelectListItem { Text = o.Name, Value = o.Id.ToString(), Selected = (searchCriteria != null && searchCriteria.ActivityTypeId.HasValue && o.Id == searchCriteria.ActivityTypeId.Value) }).ToList();
            types.Insert(0, new SelectListItem { Text = "Any activity", Value = "0" });

            var states = this.activitiesRepository.RetrieveStates()
                                        .Select(o => new SelectListItem { Text = o, Value = o, Selected = (searchCriteria != null && o == searchCriteria.State) }).ToList();
            states.Insert(0, new SelectListItem { Text = "Any state", Value = string.Empty });

            viewData["SearchFields"] = new ActivitySearchFields
            {
                ActivityTypes = types,
                States = states,
            };
        }


        private string GetCriteriaDescription(ActivitySearchCriteria searchCriteria, string separator = " | ")
        {
            StringBuilder title = new StringBuilder();
            if (searchCriteria.ActivityTypeId > 0)
            {
                title.Append(this.ActivityTypes.Where(a => a.Id == searchCriteria.ActivityTypeId).Select(a => a.PluralizedName).FirstOrDefault());
                title.Append(separator);
            }

            if (!string.IsNullOrEmpty(searchCriteria.City))
            {
                title.Append(searchCriteria.City);
                title.Append(separator);
            }

            if (!string.IsNullOrEmpty(searchCriteria.State))
            {
                title.Append(searchCriteria.State);
                title.Append(separator);
            }

            if (!string.IsNullOrEmpty(searchCriteria.Zip))
            {
                title.Append(searchCriteria.Zip);
                title.Append(separator);
            }

            title.AppendFormat(CultureInfo.CurrentUICulture, "Page {0}", searchCriteria.Page);

            return title.ToString();
        }

        public ActionResult Search(ActivitySearchCriteria searchCriteria)
        {
            if (searchCriteria.ActivityTypeId.HasValue && searchCriteria.ActivityTypeId == 0)
            {
                searchCriteria.ActivityTypeId = null;
            }

            // Validation
            var errors = new Collection<ValidationResult>();
            Validator.TryValidateObject(searchCriteria, new ValidationContext(searchCriteria, null, null), errors);
            if (errors.Any())
            {
                ModelState.AddModelError("ActivitySearchCriteria", "Please provide a search criteria.");
                return this.Index();
            }

            searchCriteria.PageSize = DefaultPageSize;
            if (searchCriteria.Page <= 0)
            {
                searchCriteria.Page = 1;
            }

            // search activities
            var activities = this.activitiesRepository.Search(searchCriteria);
            this.ViewData["PagingResult"] = activities;

            this.ViewData["SearchCriteria"] = searchCriteria;
            this.InjectActivitySearchFields(this.ViewData, searchCriteria);
            this.ViewData["CriteriaDescription"] = this.GetCriteriaDescription(searchCriteria, " | ");
            this.ViewData["KeywordsMetatag"] = this.GetCriteriaDescription(searchCriteria, ", ");
            return View("Index");
        }

    }
}